@@ -41,7 +41,7 @@ from . import cxx
 compilers = ['c++']
 
 
-def generate(env):
+def generate(env) -> None:
     """Add Builders and construction variables for g++ to an Environment."""
     static_obj, shared_obj = SCons.Tool.createObjBuilders(env)
 
