@@ -51,8 +51,8 @@ class CopytreeError(OSError):
     pass
 
 
-def scons_copytree(src, dst, symlinks=False, ignore=None, copy_function=copy2,
-                   ignore_dangling_symlinks=False, dirs_exist_ok=False):
+def scons_copytree(src, dst, symlinks: bool=False, ignore=None, copy_function=copy2,
+                   ignore_dangling_symlinks: bool=False, dirs_exist_ok: bool=False):
     """Recursively copy a directory tree, SCons version.
 
     This is a modified copy of the Python 3.7 shutil.copytree function.
@@ -229,12 +229,12 @@ def listShlibLinksToInstall(dest, source, env):
                 install_links.append((install_link, install_linktgt))
     return install_links
 
-def installShlibLinks(dest, source, env):
+def installShlibLinks(dest, source, env) -> None:
     """If we are installing a versioned shared library create the required links."""
     Verbose = False
     symlinks = listShlibLinksToInstall(dest, source, env)
     if Verbose:
-        print('installShlibLinks: symlinks={!r}'.format(StringizeLibSymlinks(symlinks)))
+        print(f'installShlibLinks: symlinks={StringizeLibSymlinks(symlinks)!r}')
     if symlinks:
         CreateLibSymlinks(env, symlinks)
     return
@@ -330,7 +330,7 @@ def add_versioned_targets_to_INSTALLED_FILES(target, source, env):
     Verbose = False
     _INSTALLED_FILES.extend(target)
     if Verbose:
-        print("add_versioned_targets_to_INSTALLED_FILES: target={!r}".format(list(map(str, target))))
+        print(f"add_versioned_targets_to_INSTALLED_FILES: target={list(map(str, target))!r}")
     symlinks = listShlibLinksToInstall(target[0], source, env)
     if symlinks:
         EmitLibSymlinks(env, symlinks, target[0])
@@ -341,7 +341,7 @@ class DESTDIR_factory:
     """ A node factory, where all files will be relative to the dir supplied
     in the constructor.
     """
-    def __init__(self, env, dir):
+    def __init__(self, env, dir) -> None:
         self.env = env
         self.dir = env.arg2nodes( dir, env.fs.Dir )[0]
 
@@ -433,7 +433,7 @@ def InstallVersionedBuilderWrapper(env, target=None, source=None, dir=None, **kw
 added = None
 
 
-def generate(env):
+def generate(env) -> None:
 
     from SCons.Script import AddOption, GetOption
     global added
@@ -504,8 +504,8 @@ def generate(env):
     except KeyError:
         env['INSTALLVERSIONEDLIB'] = copyFuncVersionedLib
 
-def exists(env):
-    return 1
+def exists(env) -> bool:
+    return True
 
 # Local Variables:
 # tab-width:4
