@@ -34,6 +34,7 @@ import SCons
 import SCons.Script
 import SCons.Tool.ninja.Globals
 from SCons.Script import GetOption
+from SCons.Util import sanitize_shell_env
 
 from .Globals import NINJA_RULES, NINJA_POOLS, NINJA_CUSTOM_HANDLERS, NINJA_DEFAULT_TARGETS, NINJA_CMDLINE_TARGETS
 from .Methods import register_custom_handler, register_custom_rule_mapping, register_custom_rule, register_custom_pool, \
@@ -84,7 +85,7 @@ def ninja_builder(env, target, source):
         with open('run_ninja_env.bat', 'w') as f:
             for key in env['ENV']:
                 f.write('set {}={}\n'.format(key, env['ENV'][key]))
-            f.write('{} -f {} %*\n'.format(NINJA_STATE.ninja_bin_path, generated_build_ninja))
+            f.write(f'{NINJA_STATE.ninja_bin_path} -f {generated_build_ninja} %*\n')
         cmd = ['run_ninja_env.bat']
 
     else:
@@ -100,14 +101,18 @@ def ninja_builder(env, target, source):
         # reproduce the output like a ninja build would
         def execute_ninja():
 
+            if env['PLATFORM'] == 'win32':
+                spawn_env = os.environ
+            else:
+                spawn_env = sanitize_shell_env(env['ENV'])
+
             proc = subprocess.Popen(cmd,
                                     stderr=sys.stderr,
                                     stdout=subprocess.PIPE,
                                     universal_newlines=True,
-                                    env=os.environ if env["PLATFORM"] == "win32" else env['ENV']
+                                    env=spawn_env
                                     )
-            for stdout_line in iter(proc.stdout.readline, ""):
-                yield stdout_line
+            yield from iter(proc.stdout.readline, "")
             proc.stdout.close()
             return_code = proc.wait()
             if return_code:
@@ -130,7 +135,7 @@ def ninja_builder(env, target, source):
         sys.stdout.write("\n")
 
 
-def options(opts):
+def options(opts) -> None:
     """
     Add command line Variables for Ninja builder.
     """
@@ -296,7 +301,7 @@ def generate(env):
     # instead return something that looks more expanded. So to
     # continue working even if a user has done this we map both the
     # "bracketted" and semi-expanded versions.
-    def robust_rule_mapping(var, rule, tool):
+    def robust_rule_mapping(var, rule, tool) -> None:
         provider = gen_get_response_file_command(env, rule, tool)
         env.NinjaRuleMapping("${" + var + "}", provider)
 
@@ -416,7 +421,7 @@ def generate(env):
     # The Serial job class is SIGNIFICANTLY (almost twice as) faster
     # than the Parallel job class for generating Ninja files. So we
     # monkey the Jobs constructor to only use the Serial Job class.
-    SCons.Job.Jobs.__init__ = ninja_always_serial
+    SCons.Taskmaster.Job.Jobs.__init__ = ninja_always_serial
 
     ninja_syntax = importlib.import_module(".ninja_syntax", package='ninja')
 
@@ -453,7 +458,7 @@ def generate(env):
     # In the future we may be able to use this to actually cache the build.ninja
     # file once we have the upstream support for referencing SConscripts as File
     # nodes.
-    def ninja_execute(self):
+    def ninja_execute(self) -> None:
 
         target = self.targets[0]
         if target.get_env().get('NINJA_SKIP'):
@@ -469,7 +474,7 @@ def generate(env):
     # date-ness.
     SCons.Script.Main.BuildTask.needs_execute = lambda x: True
 
-    def ninja_Set_Default_Targets(env, tlist):
+    def ninja_Set_Default_Targets(env, tlist) -> None:
         """
             Record the default targets if they were ever set by the user. Ninja
             will need to write the default targets and make sure not to include
